<svg width="24"
     height="24"
     fill="none"
     stroke="currentColor"
     stroke-width="2"
     stroke-linecap="round"
     stroke-linejoin="round">
    <path d="m19 19-3.5-3.5"></path>
    <circle cx="11" cy="11" r="6"></circle>
</svg>
